
#include "inc.h"

#include "minix.h"
#include "irq_arm.h"
#include "bios_arm.h"
#include "div_arm.h"
#include "mml_arm.h"
#include "ad_arm.h"
#include "gbfs.h"
#include "lib.h"
#include "rand.h"

#include "screen.h"
#include "haribote.h"
#include "main.h"


//#include "header\font\BGFont.h"


//Mode3
//BG2 

//---------------------------------------------------------------------------
//mml.c
extern ST_MML Mml;

u8 State;

//---------------------------------------------------------------------------
EWRAM_CODE int main()
{
	State = STATE_INIT;

	for(;;)
	{
		switch(State)
		{
			case STATE_INIT:  StateInit();  break;
			case STATE_RESET: StateReset(); break;
			case STATE_PLAY:  StatePlay();  break;
			case STATE_STOP:  StateStop();  break;

			default:
				DebugMessage("[Err] main State");
				break;
		}
	}

	return 0;
}
//---------------------------------------------------------------------------
EWRAM_CODE void StateInit()
{
	InitWS();
	InitLib();
	InitKey();
	InitIRQ();
	InitFont();
	InitSprite();

//	AdInit();
	MmlInit();
	ScreenImgInit();

	SetMode(MODE_3 | BG2_ENABLE | OBJ_ENABLE | OBJ_MAP_2D);

	State = STATE_RESET;
}
//---------------------------------------------------------------------------
EWRAM_CODE void InitWS()
{
	REG_WSCNT = 0x4317;
}
//---------------------------------------------------------------------------
EWRAM_CODE void InitLib()
{
	LibInit();
}
//---------------------------------------------------------------------------
EWRAM_CODE void InitKey()
{
	KeyInit();
}
//---------------------------------------------------------------------------
EWRAM_CODE void InitIRQ()
{
	REG_INTERUPT = (u32)IRQUserHandler;
	REG_IE       = IRQ_BIT_TIMER2;

	REG_TM2D     = 65536 - 2621;			// (((16*1024*1024) / 64) * 10) / 1000
	REG_TM2CNT   = TM_ENABLE | TM_FREQ_PER_64 | TM_USEIRQ;

	REG_IME      = IRQ_MASTER_ON;
}
//---------------------------------------------------------------------------
EWRAM_CODE void InitFont()
{
	ST_FONT_SRC half;		//p

	half.pDat   = (u8*)&mplus_s10rBitmap;
	half.pSheet = (u16*)&mplus_sfnt_txt;
	half.imgCx  = MPLUS_S10R_IMG_CX;
	half.cnt    = MPLUS_S10R_FONT_CNT;
	half.cx     = MPLUS_S10R_FONT_CX;
	half.cy     = MPLUS_S10R_FONT_CY;

	Mode3SetFont(NULL, &half);
}
//---------------------------------------------------------------------------
EWRAM_CODE void InitSprite()
{
	SpriteInit();

//	SpriteSetPalData16((u16*)&sprPal);
//	SpriteSetData((u16*)&sprTiles, sprTilesLen);
}
//---------------------------------------------------------------------------
EWRAM_CODE void StateReset()
{
	MmlSetStr(1, "T120O4L4 RRRRRRRR CDEFEDCR EFGAGFER CRCRCRCR L8CCDDEEFF L4EDCR");
	MmlSetStr(2, "T120O5L4 CDEFEDCR EFGAGFER CRCRCRCR L8CCDDEEFF L4EDCR RRRRRRRR");
	MmlSetStr(4, "T120L4 RBRBRBRB RBRBRBRB8B8 RBRBRBRB RBRBRBRB8B8 RB8B8RBRBBC"  );
	MmlPlay();

	State = STATE_PLAY;
}
//---------------------------------------------------------------------------
EWRAM_CODE void StatePlay()
{
	char buf[256];
	ScreenImgClear();

	Mode3DrawFontStr(16,  8, (u8*)"KAERU NO UTA", COLOR_WHITE);

	_Sprintf(buf, "ch1 =  L:%3d  F:%4d", Mml.ch1.lenCtdn, Mml.ch1.fleq);
	Mode3DrawFontStr(16, 32, (u8*)buf, COLOR_WHITE);

	_Sprintf(buf, "ch2 =  L:%3d  F:%4d", Mml.ch2.lenCtdn, Mml.ch2.fleq);
	Mode3DrawFontStr(16, 48, (u8*)buf, COLOR_WHITE);

	_Sprintf(buf, "ch4 =  L:%3d  F:%4d", Mml.ch4.lenCtdn, Mml.ch4.fleq);
	Mode3DrawFontStr(16, 64, (u8*)buf, COLOR_WHITE);


	WaitForVsync();
	ScreenImgUpdate();

	if( MmlIsPlay() == FALSE )
	{
		State = STATE_STOP;
	}
}
//---------------------------------------------------------------------------
EWRAM_CODE void StateStop()
{
	Mode3DrawFontStr(16, SCREEN_CY-22, (u8*)"PUSH A BUTTON", COLOR_WHITE);

	WaitForVsync();
	ScreenImgUpdate();
	KeyWait(KEY_A);

	State = STATE_RESET;
}




//---------------------------------------------------------------------------
EWRAM_CODE void SystemError(char* msg1, char* msg2)
{
	u16 cy = 11;

	ScreenImgFadeOut(0);
	LibMode3SetScreenBuffer((u16*)VideoBuffer);

	if(msg1 != NULL) Mode3DrawFontStr(0,  0, (u8*)msg1, COLOR_WHITE);
	if(msg2 != NULL) Mode3DrawFontStr(0, cy, (u8*)msg2, COLOR_WHITE);

	for(;;){}
}
